import { Injectable } from '@nestjs/common';
import { AppLogger } from './app.logger';
import { ConfigService } from '@nestjs/config';
import { CheckConfig } from './config';
import { Enter } from './Enter';
import { delay } from 'q';

@Injectable()
export class AppService {
  private checkConfig: CheckConfig;
  private enters: Enter[];

  constructor(private log: AppLogger, private configService: ConfigService) {
    this.log.setContext('Enter Checker');
    this.checkConfig = {
      checkInterval: configService.get('checkInterval'),
      checkPort: configService.get('checkPort'),
      checkTimeout: configService.get('checkTimeout'),
    };
    const enterDatas = configService.get('enters');
    this.enters = enterDatas.map((enterData) => new Enter(enterData));
    this.mainCheckLoop();
  }

  private async mainCheckLoop() {
    while (true) {
      await this.checkAllEnters();
      await delay(
        this.checkConfig.checkInterval - this.checkConfig.checkTimeout,
      );
    }
  }

  private async checkAllEnters() {
    this.log.log(`Start checking ${this.enters.length} enters.`);
    return await Promise.all(
      this.enters.map((enter) => this.checkEnter(enter)),
    );
  }

  private async checkEnter(enter: Enter) {
    const checkErrorMessage = await enter.check(this.checkConfig);
    if (checkErrorMessage) {
      this.log.error(`Enter ${enter.name} is bad: ${checkErrorMessage}`);
    } else {
      this.log.log(`Enter ${enter.name} is good.`);
    }
  }

  getAllEnters(includeBad: boolean, location: string) {
    let enters = this.enters;
    if (!includeBad) {
      enters = enters.filter((enter) => enter.good);
    }
    if (location) {
      enters = enters.filter((enter) => enter.location === location);
    }
    return enters;
  }

  getOneEnter(includeBad: boolean, location: string) {
    const enters = this.getAllEnters(includeBad, location);
    if (!enters.length) {
      return null;
    }
    const enterRandomList: Enter[] = [];
    for (const enter of enters) {
      for (let i = 0; i < enter.weight; ++i) {
        enterRandomList.push(enter);
      }
    }
    return enterRandomList[Math.floor(Math.random() * enterRandomList.length)];
  }
}
